function Paper_LRBFGS_p0_for_plot(n1, n2)
% Reproduce the experiments in Wen Huang's Dissertation
% Test the performance of Riemannian algorithms for the Brockett problem on
% the Stiefel manifold.
% 
% By Wen Huang

    ps = [1, 2, 4, 8];
    n1 = 64;
    n2 = 64;
    
    for j = 1 : length(ps)
        row = 0;
        [X, F, G, T, timecost, nf, ng, nR, nH, nV, nVp, ave_time, ks] = test_PhaseRetrieval(n1, n2, ps(j), 0.9, 6);
        FS{j} = F;
        GS{j} = G;
        TS{j} = T;
        KS{j} = ks;
    end
    save(['Paper_LRBFGS_p0_FSGSTSKS_' num2str(n1) '_' num2str(n2)], 'FS', 'GS', 'TS', 'KS');
    
%     fout = fopen('Paper_LRBFGS_tab.txt','w');
%     for i = 1 : size(tabmean, 1)
%         for j = 1 : size(tabmean, 2)
%             tabmeanstr = outputfloat(tabmean(i, j));
%             tabstdstr = outputfloat(tabstd(i, j));
%             if(j < size(tabmean, 2))
%                 fprintf(fout,['$' tabmeanstr '$/$' tabstdstr '$ & ']);
%             else
%                 fprintf(fout,['$' tabmeanstr '$/$' tabstdstr '$']);
%             end
%         end
%         fprintf(fout,' \\\\\n');
%     end
%     fclose(fout);
end

function str = outputfloat(x)
    if(x <= 0)
        str = '';
        return;
    end
    p = log(x)/log(10);
    p = - ceil(-p);
    x = round(x * 10^(-p) * 100);
    x = x / 100;
    strx = sprintf('%3.2f', x);
    if(p ~= 0)
        str = [strx '_{' num2str(p) '}'];
    else
        str = strx;
    end
end

function [X, F, G, T, timecost, nf, ng, nR, nH, nV, nVp, ave_time, ks] = test_PhaseRetrieval(n1, n2, p, delta, randseed)
% test file for test_PhaseRetrieval.
% 
% By Wen Huang
    fprintf('test for Phase Retrieval problem. \n');
    r = randseed;
%     r = 943
    rand('state', r);
    randn('state', r);
%     n1 = 64;
%     n2 = 64;
    global count
    count = 0;
    n = n1 * n2;
    l = 6;
    m = l * n;
    k = p;
    method = 3;
    
    fprintf('In this case, n1 = %d, n2 = %d, n = %d, l = %d. b, and x0 are chosen randomly. Random seed is %d \n', n1, n2, n, l, r);
    
    metric = 1;

    %% generate masks and a true solution.
    global iscomplex
    iscomplex = 1;
    global isfixedrank
    isfixedrank = 0;
    masks = complex(randn(n1, n2, l), iscomplex * randn(n1, n2, l));
    phase = complex(randn(n1, n2), iscomplex * randn(n1, n2));
    phase = phase / norm(phase);
    b = zeros(m, 1);
    for i = 1 : l
        zi = reshape(fft2(phase .* masks(:, :, i)) / sqrt(n1 * n2), [], 1);
        b((i - 1) * n + 1 : i * n) = conj(zi) .* zi;
    end
%     bnorm = norm(b)
%     phasenorm = norm(phase, 'fro')
    
    %%
    initialtype = 0; % 0: random, 1: true solution, 2: modification of true solution
    x0eps = 0.1;
    if((initialtype == 1 || initialtype == 2) && k ~= 1)
        fprintf('k should be 1\n');
        return;
    end
    
    if(initialtype == 0)
        [x1.U, r] = qr(complex(randn(n, k), iscomplex * randn(n, k)), 0);
%         x1.D = spdiags(rand(k, 1), 0, k, k);
        x1.D = spdiags(ones(k, 1), 0, k, k);
    elseif(initialtype == 1)
        x1.U = reshape(phase, [], 1) / norm(reshape(phase, [], 1));
        x1.D = norm(reshape(phase, [], 1))^2;
    elseif(initialtype == 2)
        temp = phase + randn(n1, n2) * x0eps;
        x1.U = reshape(temp, [], 1) / norm(reshape(temp, [], 1));
        x1.D = norm(reshape(temp, [], 1))^2;
    end
    
    if(metric == 1 || metric == 2)
        x1.U = full(x1.U * sqrt(x1.D));
        if(method == 2 || method == 5)
            params.H0 = eye(n * k);
            params.B0 = eye(n * k);
        end
    end
    
    params.x0 = x1;
    params.error = 1e-6;
    params.max_t = inf;
    params.debug = 0;
    params.manifold = 2;
    params.m = 4;
    params.alpha = 1e-4;
    params.beta = 0.999;
    params.min_Delta = 1e-8;
    params.minstepsize = eps;
    params.maxstepsize = 200;
%     params.kappa = 0.9;
%     params.retraction = Retraction;
%     params.vector_transport = VT;
    params.Delta_bar = inf;
    params.Delta0 = 1;
    params.StopCriterion = 0;
    params.restart = 0;
    if(method == 11)
        params.linesearch = 1;
    end
    times = 0;
    total_timecost = 0;
    while(total_timecost < 60 && times < 1)
        fprintf('%d: ', times);
        [X, F, G, T1, timecost, nf, ng, nR, nH, nV, nVp, ks] = driver_PhaseRetrieval(b, masks, method, params, phase, metric, delta, 0, 0);
        if(times == 0)
            T = T1;
        else
            T = T + T1;
        end
        total_timecost = total_timecost + timecost;
        times = times + 1;
    end
    if(method == 13)
        return;
    end
    T = T / times;
    ave_time = total_timecost / times;
    fprintf('average time cost of %d times is %f. \n', times, total_timecost / times);
end
