function paper_gold_ball(tau, maskstype, method, ls)
% maskstype = 0: gaussian masks
% maskstype = 1: binary masks
% method = 0: LRBFGS
% method = 1: LRFISTA

    if(maskstype == 1) %binary
%         ls = 32;
        delta = 0.9;
        err = 1e-6;
    else %gaussian
%         ls = 6;
        delta = 0.9;
        err = 1e-6;
    end
    ratio = 0.2;
    
    load('goldballs_256.mat');
%     x0 = x0(1:64, 1:64);
%     randn('state', 0);
%     x0 = complex(randn(16, 16), randn(16, 16));
    kappa = 0;
    if(method == 0)
        for i = 1 : length(ls)
            [X, F, G, T, timecost, nf, ng, nR, nH, nV, nVp, ave_time, LRBFGSmse, snr] = test_PhaseRetrieval(ls(i), tau, 1, x0, kappa, err, maskstype, delta); %Gaussian
            LRBFGSend = reshape(X{end}.U, size(x0));
            LRBFGSt = timecost;
        end
        save(['GlodBall_LRBFGS_' num2str(tau) '_' num2str(maskstype) '_' num2str(ls(1)) '.mat'], 'x0', 'LRBFGSend', 'snr', 'LRBFGSmse', 'LRBFGSt');
    end
%     if(method == 1)
%         for i = 1 : length(ls)
%             [iter, nf, ng, f_final, LRFISTAmse, ave_time, snr, x] = test_TFOCS_PhaseRetrieval(ls(i), tau, 1, x0, kappa, err, maskstype, 60*60); %Gaussian
%             LRFISTAend = reshape(x(:, 1), size(x0));
%             LRFISTAt = ave_time;
%         end
%         save(['GlodBall_LRFISTA_' num2str(tau) '_' num2str(maskstype) '_' num2str(ls(1)) '.mat'], 'x0', 'LRFISTAend', 'snr', 'LRFISTAmse', 'LRFISTAt');
%     end
end

function str = outputfloat(x)
    if(x <= 0)
        str = '';
        return;
    end
    p = log(x)/log(10);
    p = - ceil(-p);
    x = round(x * 10^(-p) * 100);
    x = x / 100;
    strx = sprintf('%3.2f', x);
    if(p ~= 0)
        str = [strx '_{' num2str(p) '}'];
    else
        str = strx;
    end
end

function [X, F, G, T, timecost, nf, ng, nR, nH, nV, nVp, ave_time, mse, snr] = test_PhaseRetrieval(l, tau, isgaussian, imagegb, kappa, scerr, maskstype, delta)
% test file for test_PhaseRetrieval.
% 
% By Wen Huang
    fprintf('test for Phase Retrieval problem. \n');
    r = 1;
%     r = 943
    rand('state', r);
    randn('state', r);
    [n1, n2] = size(imagegb);
    global count
    count = 0;
    n = n1 * n2;
%     l = 6;
    m = l * n;
    k = 8;
    method = 3;
    
    fprintf('In this case, n1 = %d, n2 = %d, n = %d, l = %d. b, and x0 are chosen randomly. Random seed is %d \n', n1, n2, n, l, r);
    
    metric = 1;

    %% generate masks and a true solution.
    global iscomplex
    iscomplex = 1;
    global isfixedrank
    isfixedrank = 0;
    if(maskstype == 0)
        masks = complex(randn(n1, n2, l), iscomplex * randn(n1, n2, l));
    else
        masks = (randn(n1, n2, l) > 0.5) + 0;
        masks(:, :, 1) = ones(n1, n2, 1);
    end
%     phase = complex(randn(n1, n2), iscomplex * randn(n1, n2));
%     phase = phase / norm(phase);
    phase = imagegb / norm(imagegb);
    b = zeros(m, 1);
    btrue = zeros(m, 1);
    for i = 1 : l
        zi = reshape(fft2(phase .* masks(:, :, i)) / sqrt(n1 * n2), [], 1);
        btrue((i - 1) * n + 1 : i * n) = conj(zi) .* zi;
        b((i - 1) * n + 1 : i * n) = conj(zi) .* zi;
    end
    noiseb = randn(m, 1);
    expectsnr = tau;
    normnoise = norm(btrue) / (10^(expectsnr / 10));
    noiseb = normnoise * noiseb / norm(noiseb);
    b = b + noiseb;
    
    snr = 10 * log(norm(btrue, 'fro') / norm(btrue - b, 'fro')) / log(10)
%     bnorm = norm(b)
%     phasenorm = norm(phase, 'fro')
    
    %%
    initialtype = 0; % 0: random, 1: true solution, 2: modification of true solution
    x0eps = 0.1;
    if((initialtype == 1 || initialtype == 2) && k ~= 1)
        fprintf('k should be 1\n');
        return;
    end
    
    if(initialtype == 0)
        [x1.U, r] = qr(complex(randn(n, k), iscomplex * randn(n, k)), 0);
%         x1.D = spdiags(rand(k, 1), 0, k, k);
        x1.D = spdiags(ones(k, 1), 0, k, k);
    elseif(initialtype == 1)
        x1.U = reshape(phase, [], 1) / norm(reshape(phase, [], 1));
        x1.D = norm(reshape(phase, [], 1))^2;
    elseif(initialtype == 2)
        temp = phase + randn(n1, n2) * x0eps;
        x1.U = reshape(temp, [], 1) / norm(reshape(temp, [], 1));
        x1.D = norm(reshape(temp, [], 1))^2;
    end
    
    if(metric == 1 || metric == 2)
        x1.U = full(x1.U * sqrt(x1.D));
        if(method == 2 || method == 5)
            params.H0 = eye(n * k);
            params.B0 = eye(n * k);
        end
    end
    
    params.x0 = x1;
    params.error = scerr;
    params.max_t = inf;
%     params.max_t = 10;
    params.debug = 0;
    params.manifold = 2;
    params.m = 4;
    params.alpha = 1e-4;
    params.beta = 0.999;
    params.min_Delta = 1e-8;
    params.minstepsize = eps;
    params.maxstepsize = 20000;
%     params.kappa = 0.9;
%     params.retraction = Retraction;
%     params.vector_transport = VT;
    params.Delta_bar = inf;
    params.Delta0 = 1;
    params.StopCriterion = 0;
    params.restart = 0;
    params.keepnumX = 2;
    if(method == 11)
        params.linesearch = 1;
    end
    times = 0;
    total_timecost = 0;
    while(total_timecost < 60 && times < 1)
        fprintf('%d: ', times);
        [X, F, G, T1, timecost, nf, ng, nR, nH, nV, nVp] = driver_PhaseRetrieval(b, masks, method, params, phase, metric, delta, 0, kappa);
        if(times == 0)
            T = T1;
        else
            T = T + T1;
        end
        total_timecost = total_timecost + timecost;
        times = times + 1;
    end
    if(method == 13)
        return;
    end
    mse = MSE_error(X{end}.U, reshape(phase, [], 1));
    T = T / times;
    ave_time = total_timecost / times;
    fprintf('average time cost of %d times is %f. \n', times, total_timecost / times);
end

function mse = MSE_error(x, phase)
    X1 = [real(x), imag(x)];
    X2 = [real(phase), imag(phase)];
    [U, S, V] = svd(X2'*X1);
    O = V * U';
    mse = norm(X1 * O - X2, 'fro') / norm(X2, 'fro');
end

function [iter, nf, ng, f_final, mse, ave_time, snr, x] = test_TFOCS_PhaseRetrieval(l, tau, isgaussian, imagegb, kappa, scerr, maskstype, max_time)
%% Now, add in constraints
    r = rand();
%     r = floor(r * 10000)
    r = 1;
    rand('state', r);
    randn('state', r);
    [n1, n2] = size(imagegb);
    n = n1 * n2;
%     l = 6;
    m = l * n;
    k = 1;

    %% generate masks and a true solution.
    global iscomplex
    iscomplex = 1;
    if(maskstype == 0)
        masks = complex(randn(n1, n2, l), iscomplex * randn(n1, n2, l));
    else
        masks = (randn(n1, n2, l) > 0.5) + 0;
        masks(:, :, 1) = ones(n1, n2, 1);
    end
%     phase = complex(randn(n1, n2), iscomplex * randn(n1, n2));
%     phase = phase / norm(phase);
    phase = imagegb / norm(imagegb);
    b = zeros(m, 1);
    btrue = zeros(m, 1);
    for i = 1 : l
        zi = reshape(fft2(phase .* masks(:, :, i)) / sqrt(n1 * n2), [], 1);
        btrue((i - 1) * n + 1 : i * n) = conj(zi) .* zi;
        b((i - 1) * n + 1 : i * n) = conj(zi) .* zi;
    end
    noiseb = randn(m, 1);
    expectsnr = tau;
    normnoise = norm(btrue) / (10^(expectsnr / 10));
    noiseb = normnoise * noiseb / norm(noiseb);
    b = b + noiseb;
    snr = 10 * log(norm(btrue, 'fro') / norm(btrue - b, 'fro')) / log(10)
%     bnorm = norm(b)
%     phasenorm = norm(phase, 'fro')
    
    initialtype = 0; % 0: random, 1: true solution, 2: modification of true solution
    x0eps = 0.1;
    if((initialtype == 1 || initialtype == 2) && k ~= 1)
        fprintf('k should be 1\n');
        return;
    end
    
    if(initialtype == 0)
        [x1.U, r] = qr(complex(randn(n, k), iscomplex * randn(n, k)), 0);
%         x1.D = spdiags(rand(k, 1), 0, k, k);
        x1.D = spdiags(ones(k, 1), 0, k, k);
    elseif(initialtype == 1)
        x1.U = reshape(phase, [], 1) / norm(reshape(phase, [], 1));
        x1.D = norm(reshape(phase, [], 1))^2;
    elseif(initialtype == 2)
        temp = phase + randn(n1, n2) * x0eps;
        x1.U = reshape(temp, [], 1) / norm(reshape(temp, [], 1));
        x1.D = norm(reshape(temp, [], 1))^2;
    end
    x0 = full(x1.U * sqrt(x1.D));

    smoothF = @(x)f_gf(x, masks, b, kappa);
    
    opts = [];
    opts.tol        = scerr;
    opts.maxits     = 10000;
    opts.alg = 'N83_PR';
    opts.max_time = max_time;
%     opts.stopCrit = 2;
%     opts.stopFcn = @stopFcn;
%     opts.err = 1e-6;
    
    times = 0;
    total_timecost = 0;
    while(total_timecost < 60 && times < 1)
        fprintf('times:%d, initial p:%d \n', times, k);
        [ x, out, optsOut ] = tfocs(smoothF, [], [], x0, opts );
        timecost = optsOut.timecost;
        total_timecost = total_timecost + timecost;
        times = times + 1;
    end
    ave_time = total_timecost / times;
    iter = optsOut.iter;
    nf = optsOut.nf;
    ng = optsOut.ng;
    f_final = f_gf(x(:, 1), masks, b, 0);
    mse = MSE_error(x(:, 1), reshape(phase, [], 1));
return;
end

function stop = stopFcn(f, x)
    stop = 0;% f < 1e-10;
return;
end

function [f, gf] = f_gf(x, masks, b, kappa)
    [n1, n2, l] = size(masks);
    n = n1 * n2;
    k = size(x, 2);
    m = l * n;
    D = zeros(m, 1);
    ZY = zeros(m, k);
    for i = 1 : l
        sZqi = zeros(n, 1);
        for j = 1 : k
            temp = reshape(fft2(reshape(x(:, j), n1, n2) / sqrt(n1 * n2) .* masks(:, :, i)), [], 1);
            ZY((i - 1) * n + 1 : i * n, j) = reshape(temp, [], 1);
            sZqi = sZqi + conj(temp) .* temp;
        end
        D((i - 1) * n + 1 : i * n) = reshape(sZqi, [], 1) - b((i - 1) * n + 1 : i * n);
    end
    f = norm(D)^2 / norm(b, 'fro')^2;
    f = f + kappa * norm(x, 'fro')^2;%% add penelty term
    gf = @(v)gfv(v, D, masks, b, kappa);
return
end

function output = gfv(v, D, masks, b, kappa)
    [n1, n2, l] = size(masks);
    n = n1 * n2;
    s = size(v, 2);
    m = length(D);
    Zv = zeros(m, s);    
    for i = 1 : l
        for j = 1 : s
            temp = reshape(fft2(reshape(v(:, j), n1, n2) / sqrt(n1 * n2) .* masks(:, :, i)), [], 1);
            Zv((i - 1) * n + 1 : i * n, j) = reshape(temp, [], 1);
        end
    end
    
    DZv = spdiags(D, 0, m, m) * Zv;
    %% Z^H *DZv
    Y = zeros(n, s);
    for i = 1 : l
        temp = zeros(n, s);
        for j = 1 : s
            temp(:, j) = reshape(ifft2(reshape(DZv((i - 1) * n + 1 : i * n, j) * sqrt(n1 * n2), n1, n2)), [], 1);
        end
        temp = conj(spdiags(reshape(masks(:, :, i), [], 1), 0, n, n)) * temp;
        Y = Y + temp;
    end
    output = 2 * Y / norm(b, 'fro')^2 + kappa * v;
return
end

