function [ x, out, opts ] = tfocs_N83( smoothF, affineF, projectorF, x0, opts )
% TFOCS_N83 Nesterov's 1983 accelerated method; also by Beck and Teboulle 2005 (FISTA).
% [ x, out, opts ] = tfocs_N83( smoothF, affineF, nonsmoothF, x0, opts )
%   Implements Nesterov's 1983 method.
%   A variety of calling sequences are supported; type 
%      help tfocs_help
%   for a full explanation.

% Initialization
alg = 'N83';
algorithm = 'Nesterov''s 1983 single-projection method';
alpha = 0; beta = 0; mu = 0; L = 0; % Do not remove: necessary because of a MATLAB quirk
tfocs_initialize
if nargin == 0, return; end
tic
outtimes = 0;
[n, k] = size(x);
temp = z;
clear('z');
z.U = temp;
z.D = eye(size(z.U, 2));
while true,
    fprintf('outtimes:%d, f:%e \n', outtimes, f_x);
    outtimes = outtimes + 1;
      x_old =   x;
%     A_x_old = A_x;
      z_old =   z;
%     A_z_old = A_z;
    
    % The backtracking loop
    L_old      = L;
   	L          = L * alpha;
    theta_old  = theta;
    times = 0;
    while true,
%         fprintf('times:%d \n', times);
        times = times + 1;
        % Acceleration
        theta = advance_theta( theta_old, L, L_old );
%         theta
        % Next iterate
        if theta < 1,
%             y   = ( 1 - theta ) *   x_old + theta *   z_old;
            y = low_rank_addition(( 1 - theta ), x_old , theta, z_old, k);
%             A_y = ( 1 - theta ) * A_x_old + theta * A_z_old;
%             A_y = low_rank_addition(( 1 - theta ), A_x_old , theta, A_z_old, k);
            f_y = Inf;% g_Ay = []; 
            g_y = [];
        end
        % Compute function values
        if isempty( g_y ),
%             if isempty( g_Ay ), 
%                 [ f_y, g_Ay ] = apply_smooth( A_y ); 
%             end
            [ f_y, g_y ] = apply_smooth( y ); 
%             g_y = apply_linear( g_Ay, 2 ); % identity for PR
%             f_y
        end

        % Standard gradient
%         [ C_x, x ] = apply_projector( y - (1/L) * g_y, 1/L ); % identity for PR
% tic
%         temp = low_rank_add_gfv(1, y, - (1/L), g_y, k);
% h1 = toc
% ttttemp = low_rank_add_gfv(1, y, - (1/L/10), g_y);
% fprintf('\t t:%e, fy:%e, fx:%e, ftest:%e\n', 1/L, apply_smooth(y), apply_smooth(temp), apply_smooth(ttttemp));%%---

%         [ C_x, x ] = apply_projector( temp, 1/L );
        x = low_rank_add_gfv(1, y, - (1/L), g_y, k);
%         C_x
%         x
%         A_x = apply_linear( x, 1 ); % identity for PR
%         A_x
        f_x = Inf; %g_Ax = []; 
        g_x = [];
%         tfocs_backtrack

        % TFOCS_BACKTRACK
        % Backtracking helper script.

        % Quick exit for no backtracking
        if beta >= 1, break; end % SRB changing == to >=

        % Quick exit if no progress made
        xy = low_rank_substraction(1, x, -1, y, 2 * k);
        temp = xy.U' * xy.U;
        xy_sq = real(trace(xy.D * temp * xy.D' * temp));
%         xy_sq = tfocs_normsq( xy' * xy );
        if xy_sq == 0, localL = Inf; 
%             fprintf('\t no progress made\n'); 
            break; 
        end

        % Compute Lipschitz estimate
        if backtrack_simple,
            if isinf( f_x ),
%                 f_x = apply_smooth( A_x );
                f_x = apply_smooth( x );
%                 f_x
            end
        %     q_x = f_y + tfocs_dot( xy, g_y ) + 0.5 * L * xy_sq;
% tic
            first_order = inpro_gf_v(g_y, xy, n, k);
            q_x = f_y + first_order + 0.5 * L * xy_sq;
% h2 = toc

% xx = low_rank_add_gfv(1, y, - 1e-8, g_y, k);
% f_xx = apply_smooth( xx );
% xxy = low_rank_substraction(1, xx, -1, y, 2 * k);
% h1 = norm(xx * xx' - y * y' - xxy.U * xxy.D * xxy.U') / norm(xxy.U * xxy.D * xxy.U')
% h2 = (f_xx - f_y) / 1e-8
% h3 = inpro_gf_v(g_y, xxy, n, k)

            fprintf('\t f_x - f_y:%e, inpro:%e, second_order:%e\n', f_x - f_y, first_order, 0.5 * L * xy_sq);%%---
            localL = L + 2 * max( f_x - q_x, 0 ) / xy_sq;
            fprintf('\t L:%e, localL:%e, f_x:%e, q_x:%e, f_y:%e\n', L, localL, f_x, q_x, f_y);%%---
            backtrack_simple = abs( f_y - f_x ) >= backtrack_tol * max( abs( f_x ), abs( f_y ) );
fprintf('\t diff_f:%e, fx:%e, fy:%e, backtrack_simple:%d, backtrack_tol:%e\n', abs( f_y - f_x ), f_x, f_y, backtrack_simple, backtrack_tol);%%---
        else
            if isempty( g_x ),
                [ f_x, g_x ] = apply_smooth( x );
            end
        %     localL = 2 * tfocs_dot( A_x - A_y, g_Ax - g_Ay ) / xy_sq;
%             A_x_A_y = low_rank_substraction(1, A_x, -1, A_y, k);
%             localL = 2 * inpro_gf1addgf2_v(1, g_Ax, -1, g_Ay, A_x_A_y) / xy_sq;
            localL = 2 * inpro_gf1addgf2_v(1, g_x, -1, g_y, xy) / xy_sq;
        end

        % Exit if Lipschitz criterion satisfied, or if we hit Lexact
        backtrack_steps = backtrack_steps + 1;
        if localL <= L || L >= Lexact,
%             fprintf('\t Lipschitz criterion satisfied: localL:%e, L:%e\n', localL, L); 
            break; 
        end
% localL
        if ~isinf( localL ), 
            L = min( Lexact, localL );
        elseif isinf( localL ), localL = L; end
% L1 = L%%----
        L = min( Lexact, max( localL, L / beta ) );
% L2 = L%%---
        % TFOCS v1.3 by Stephen Becker, Emmanuel Candes, and Michael Grant.
        % Copyright 2013 California Institute of Technology and CVX Research.
        % See the file LICENSE for full license information.

    end
%     fprintf('total inn-iter:%d \n', times);
    
    % Collect data, evaluate stopping criteria, and print status
    tfocs_iterate_PR

    % Reversed z update. This step is skipped if restart occurs
    if theta == 1,
        z.U   =   x;
        z.D = eye(size(z.U, 2));
%         A_z = A_x;
%         C_z = C_x;
    else
%         z   = (1/theta) * x   + ( 1 - 1/theta ) *   x_old;
% fprintf('z: 1/theta:%e, 1-1/theta:%e\n', 1/theta, 1-1/theta);
        z = low_rank_substraction((1/theta), x , ( 1 - 1/theta ), x_old, 2 * k);
%         A_z = (1/theta) * A_x + ( 1 - 1/theta ) * A_x_old;
%         A_z = low_rank_addition((1/theta), A_x, ( 1 - 1/theta ), A_x_old, k);
%         C_z = Inf;
    end    
end
t = toc;
fprintf('timecost:%f \n', t);
% Final processing
tfocs_cleanup

% TFOCS v1.3 by Stephen Becker, Emmanuel Candes, and Michael Grant.
% Copyright 2013 California Institute of Technology and CVX Research.
% See the file LICENSE for full license information.

function output = low_rank_addition(a, x, b, y, k)
    if(a < 0 || b < 0)
        error('a and b must be nonnegative!');
    end
    [n, k1] = size(x);
    k2 = size(y, 2);
    Z = [x, y.U];
    [Q, R] = qr(Z, 0);
    M = R * diag([ones(k1, 1) * a; diag(y.D) * b]) * R';
    [U, D] = eig(M);
    D = real(D);
%     D = max(D, 0);
%     output = Q * U * sqrt(D);
    [d, idx] = sort(diag(D), 'descend');
    output = Q * U(:, idx(1:k)) * diag(sqrt(max(d(1:k), 0)));

%     output = Q * U(:, idx(1:k)) * diag(sqrt(d(1:k)));
%     output = Q * U * sqrt(D);
% h4 = norm(a * x * x' + b * y * y' - output * output') / norm(output * output')
% h5 = norm(a * x * x' + b * y * y' - Q * U * D * U' * Q') / norm(Q * U * D * U' * Q')
% h5 = norm(a * x * x' + b * y * y' - Q * U * D * U' * Q') / norm(Q * U * D * U' * Q')
% h6 = norm(a * x * x' + b * y * y' - Q * M * Q') / norm(Q * M * Q')
% h7 = norm(a * x * x' + b * y * y' - Z * diag([ones(k1, 1) * a; ones(k2, 1) * b]) * Z') / norm(Z * diag([ones(k1, 1) * a; ones(k2, 1) * b]) * Z')
% h8 = norm(Z - Q * R) / norm(Z)

%     [n, k] = size(x);
%     M = a * x * x' + b * y * y';
%     M = (M + M') / 2;
%     if(k == n)
%         [U, D] = eig(M);
%     else
%         [U, D] = eigs(M, k, 'lr');
%     end
%     D = max(D, 0);
%     output11 = U * sqrt(D);
%     tt1 = norm(output * output' - output11 * output11') / norm(output * output')%%---
return


function output = low_rank_substraction(a, x, b, y, k)
    if(a < 0 || b > 0)
        error('a must be nonnegative and b must be nonpositive');
    end
    [n, k1] = size(x);
    k2 = size(y, 2);
    Z = [x, y];
    [Q, R] = qr(Z, 0);
    M = R * diag([ones(k1, 1) * a; ones(k2, 1) * b]) * R';
    [U, D] = eig(M);
    D = real(D);
    output.U = Q * U;
    output.D = D;
% h4 = norm(a * x * x' + b * y * y' - output.U * output.D * output.U') / norm(output.U * output.D * output.U')

return


function output = low_rank_add_gfv(a, x, b, gf, k)
    n = size(x, 1);
    func = @(v) a * x * (x' * v) + b * gf(v);
    opts.isreal = false;
    opts.issym = 1;
    [U, D] = eigs(func, n, k, 'lr', opts);
%     diag(D)
    D = real(D);
    D = max(D, 0);
    output = U * sqrt(D);
    
%     [n, k] = size(x);
%     temp = gf(eye(n));
%     M = a * x * x' + b * temp;
%     M = (M + M') / 2;
%     if(k == n)
%         [U, D] = eig(M);
%     else
%         [U, D] = eigs(M, k, 'lr');
%     end
%     D = max(D, 0);
%     output11 = U * sqrt(D);
%     tt2 = norm(output * output' - output11 * output11') / norm(output * output')%%---
return

function output = inpro_gf_v(gf, eta, n, k)
%     opts.isreal = false;
%     opts.tol = 1e-2;
%     opts.issym = 1;
% %     opts.v0 = rand(n, 1) * 0.01;
%     [U, D] = eigs(gf, n, k, 'lr', opts);
%     diag(D)
%     D = real(D);
%     gfk = U * sqrt(D);
%     temp = gfk' * eta;
%     output = real(temp(:)' * temp(:));
    
    temp = gf(eta.U) * eta.D';
    output = real(eta.U(:)' * temp(:));
return

function output = inpro_gf1addgf2_v(a, gf1, b, gf2, v)
    temp = (a * gf1(v.U) + b * gf2(v.U)) * v.D;
    output = real(v.U(:)' * temp(:));
return

function output = gfv1_add_gfv2(a, gf1, b, gf2, n, k)
    func = @(v) a * gf1(v) + b * gf2(v);
    opts.isreal = false;
    [U, D] = eigs(func, n, k, 'lr', opts);
    D = real(D);
    output = U * sqrt(D);

%     [n, k] = size(x);
%     temp1 = a * gf1(eye(n));
%     temp2 = b * gf2(eye(n));
%     M = temp1 + temp2;
%     M = (M + M') / 2;
%     if(k == n)
%         [U, D] = eig(M);
%     else
%         [U, D] = eigs(M, k, 'lr');
%     end
%     D = max(D, 0);
%     D2 = diag(D)%%---
%     output11 = U * sqrt(D);
%     tt3 = norm(output * output' - output11 * output11') / norm(output * output')%%---
return
